from pyroll.core import RollPass, Hook
from pyroll.integral_thermal.helper import mean_temperature, mean_density, mean_specific_heat_capacity

RollPass.deformation_heat_efficiency = Hook[float]()
"""Efficiency of heat generation through deformation. 1 means that all forming energy is dissipated as heat, 0 that all energy is saved in microstructure."""

RollPass.temperature_change_by_contact = Hook[float]()
"""Get the change in temperature by contact transfer within the roll pass."""

RollPass.temperature_change_by_deformation = Hook[float]()
"""Get the change in temperature by deformation heat within the roll pass."""

RollPass.temperature_change = Hook[float]()
"""Get the change in temperature within the roll pass."""

RollPass.Roll.contact_heat_transfer_coefficient = Hook[float]()
"""Get the heat transfer coefficient for contact of rolls and workpiece."""


@RollPass.Roll.contact_heat_transfer_coefficient
def default_contact_heat_transfer_coefficient(self: RollPass.Roll):
    return 6e3


@RollPass.Roll.temperature
def default_roll_temperature(self: RollPass.Roll):
    return 293.15


@RollPass.OutProfile.temperature
def out_temperature(self: RollPass.OutProfile):
    return self.roll_pass.in_profile.temperature + self.roll_pass.temperature_change


@RollPass.deformation_heat_efficiency
def default_deformation_heat_efficiency(self: RollPass):
    return 0.95


@RollPass.temperature_change_by_contact
def temperature_change_by_contact(self: RollPass):
    return -(
            (
                    self.roll.contact_heat_transfer_coefficient
                    * (mean_temperature(self) - self.roll.temperature)
                    * self.duration
                    * self.contact_area
            ) / (
                    mean_density(self)
                    * mean_specific_heat_capacity(self)
                    * self.volume
            )
    )


@RollPass.temperature_change_by_deformation
def temperature_change_by_deformation(self: RollPass):
    deformation_resistance = (
        self.deformation_resistance
        if self.has_value("deformation_resistance")
        else (self.in_profile.flow_stress + 2 * self.out_profile.flow_stress) / 3
    )
    return (
            (
                    self.deformation_heat_efficiency
                    * deformation_resistance
                    * self.strain
            ) / (
                    mean_density(self)
                    * mean_specific_heat_capacity(self)
            )
    )


@RollPass.temperature_change
def temperature_change(self: RollPass):
    return self.temperature_change_by_contact + self.temperature_change_by_deformation



import numpy as np
from pyroll.core import Unit, PassSequence
from pyroll.report import hookimpl, utils


@hookimpl(specname="unit_plot")
def temperatures_plot(unit: Unit):
    if isinstance(unit, PassSequence):
        """Plot the temperatures of all profiles"""
        fig, ax = utils.create_sequence_plot(unit)
        ax.set_ylabel(r"temperature $T$")
        ax.set_title("Mean Profile Temperatures")

        units = list(unit)
        if len(units) > 0:
            def gen_seq():
                yield -0.5, units[0].in_profile.temperature
                for i, u in enumerate(units):
                    yield i + 0.5, u.out_profile.temperature

            x, y = np.transpose(
                list(gen_seq())
            )

            ax.plot(x, y, marker="x")

            return fig



import importlib.util

from . import roll_pass
from . import transport

VERSION = "2.1.0"

REPORT_INSTALLED = bool(importlib.util.find_spec("pyroll.report"))

if REPORT_INSTALLED:
    from . import report
    from pyroll.report import plugin_manager

    plugin_manager.register(report)

from pyroll.core import root_hooks, Unit

root_hooks.add(Unit.OutProfile.temperature)



from pyroll.core import Transport, Hook

from pyroll.integral_thermal.helper import mean_temperature, mean_density, mean_specific_heat_capacity

Transport.cooling_water_temperature = Hook[float]()
"""Get the temperature of the cooling water."""

Transport.convection_heat_transfer_coefficient = Hook[float]()
"""Get the heat transfer coefficient for contact of rolls and workpiece."""

Transport.cooling_heat_transfer_coefficient = Hook[float]()
"""Get the heat transfer coefficient for contact of rolls and workpiece."""

Transport.relative_radiation_coefficient = Hook[float]()
"""Get the heat transfer coefficient for contact of rolls and workpiece."""

Transport.temperature_change_by_convection = Hook[float]()
"""Get the change in temperature by convection within the transport."""

Transport.temperature_change_by_cooling = Hook[float]()
"""Get the change in temperature by cooling within the transport."""

Transport.temperature_change_by_radiation = Hook[float]()
"""Get the change in temperature by radiation within the transport."""

Transport.temperature_change = Hook[float]()
"""Get the change in temperature within the transport."""

stefan_boltzmann_coefficient = 5.670374419e-8


@Transport.convection_heat_transfer_coefficient
def default_convection_heat_transfer_coefficient(self: Transport):
    return 15


@Transport.cooling_heat_transfer_coefficient
def default_cooling_heat_transfer_coefficient(self: Transport):
    return 150


@Transport.relative_radiation_coefficient
def default_relative_radiation_coefficient(self: Transport):
    return 0.8


@Transport.environment_temperature
def default_environment_temperature(self: Transport):
    """Backup atmosphere temperature at 293.15 K."""
    return 293.15


@Transport.temperature_change_by_convection
def temperature_change_by_convection(self: Transport):
    if not self.has_value("environment_temperature"):
        return 0

    return -(
            (
                    self.convection_heat_transfer_coefficient
                    * (mean_temperature(self) - self.environment_temperature)
                    * self.in_profile.cross_section.length
                    * self.duration
            ) / (
                    self.in_profile.cross_section.area
                    * mean_density(self)
                    * mean_specific_heat_capacity(self)
            )
    )


@Transport.temperature_change_by_cooling
def temperature_change_by_cooling(self: Transport):
    if not self.has_value("cooling_water_temperature"):
        return 0

    return -(
            (
                    self.cooling_heat_transfer_coefficient
                    * (mean_temperature(self) - self.cooling_water_temperature)
                    * self.in_profile.cross_section.length
                    * self.duration
            ) / (
                    self.in_profile.cross_section.area
                    * mean_density(self)
                    * mean_specific_heat_capacity(self)
            )
    )


@Transport.temperature_change_by_radiation
def temperature_change_by_radiation(self: Transport):
    if not self.has_value("environment_temperature"):
        return 0

    return -(
            (
                    self.relative_radiation_coefficient
                    * stefan_boltzmann_coefficient
                    * (mean_temperature(self) ** 4 - self.environment_temperature ** 4)
                    * self.in_profile.cross_section.length
                    * self.duration
            ) / (
                    self.in_profile.cross_section.area
                    * mean_density(self)
                    * mean_specific_heat_capacity(self)
            )
    )


@Transport.temperature_change
def temperature_change(self: Transport):
    return (
            self.temperature_change_by_convection
            + self.temperature_change_by_cooling
            + self.temperature_change_by_radiation
    )


@Transport.OutProfile.temperature
def out_temperature(self: Transport.OutProfile):
    return self.transport.in_profile.temperature + self.transport.temperature_change



from pyroll.core import Unit


def mean_temperature(unit: Unit):
    return (unit.in_profile.temperature + unit.out_profile.temperature) / 2


def mean_density(unit: Unit):
    return (unit.in_profile.density + unit.out_profile.density) / 2


def mean_specific_heat_capacity(unit: Unit):
    return (unit.in_profile.specific_heat_capacity + unit.out_profile.specific_heat_capacity) / 2
